<?php

class payout
{
	private $transaction = array();
	public $error = array( 'msg' => null, 'code' => null );
	private $client = false;
	private $productive;
	private $payoutResult;
	private $url = array( 'test' => 'https://soatest.paysafecard.com/psc/services/PscService?wsdl', 'productive' =>
			'https://soa.paysafecard.com/psc/services/PscService?wsdl' );
	private $wsdl = array();
	public function __construct( $username, $password, $productive = false )
	{
		$this->transaction['username'] = $username;
		$this->transaction['password'] = $password;
		$this->productive = $productive;
	}
	public function setTransaction( $amount, $currency, $ptid = '', $utcOffset = '+01:00' )
	{
		$this->transaction['ptid'] = ( $ptid ) ? $ptid : $this->generatePtid();
		$this->transaction['amount'] = number_format(str_replace( ',', '.', $amount ),2,'.','');
		$this->transaction['currency'] = strtoupper( $currency );
		$this->transaction['utcOffset'] = $utcOffset;
	}
	public function setCustomer( $merchantClientId, $customerId, $customerIdType = 'EMAIL' )
	{
		$this->transaction['customerIdType'] = $customerIdType;
		$this->transaction['customerId'] = $customerId;
		$this->transaction['merchantClientId'] = $merchantClientId;
	}
	public function setCustomerDetails( $firstName, $lastName, $dateOfBirth )
	{
		$this->transaction['customerDetailsBasic'] = array(
			'firstName' => $firstName,
			'lastName' => $lastName,
			'dateOfBirth' => $dateOfBirth );
	}
	public function payout( $validationOnly = true )
	{
		$this->transaction['validationOnly'] = $validationOnly;
		$this->client = $this->connect();
		$result = $this->client->payout( $this->transaction );
		return $this->payoutResult( $result );
	}
	private function payoutResult( $result )
	{
		$this->payoutResult = array(
			'status' => ( $result->PayoutReturn->errorCode == 0 && $result->PayoutReturn->resultCode == 0 ) ? true : false,
			'errorMsg' => ( $result->PayoutReturn->errorCode == 0 && $result->PayoutReturn->resultCode == 0 ) ? false : $this->setError
				( $result->PayoutReturn->errorCode ),
			'ptid' => $result->PayoutReturn->ptid,
			'transactionDetails' => array(
				'requestedCurrency' => $result->PayoutReturn->requestedCurrency,
				'requestedAmount' => $result->PayoutReturn->requestedAmount,
				'payedInCurrency' => ( isset( $result->PayoutReturn->payedInCurrency ) ) ? $result->PayoutReturn->payedInCurrency : null, 
				'payedAmount' => ( isset( $result->PayoutReturn->payedAmount ) ) ? $result->PayoutReturn->payedAmount : null ), 
			'codes' => array(
				'resultCode' => $result->PayoutReturn->resultCode,
				'errorCode' => $result->PayoutReturn->errorCode,
				'errorCodeDescription' => $result->PayoutReturn->errorCodeDescription ),
			);
		return $this->payoutResult['status'];
	}
	public function getPayoutState( $username = null, $password = null, $mid = null )
	{
		if ( !$this->client )
		{
			$this->client = $this->connect();
		}
		if ( !$username )
		{
			$username = $this->transaction['username'];
			$password = $this->transaction['password'];
		}
		$data = array( 'username' => $username, 'password' => $password );
		$result = $this->client->getPayoutState( $data );
		return $this->payoutStateResult( $result->getPayoutStateReturn,($mid)?$mid:false);
	}
	private function payoutStateResult( $stateResult,$mid=false )
	{
		if ( $stateResult->resultCode != 0 or $stateResult->errorCode != 0 )
		{
			return array(
				'status' => false,
				'resultCode' => $stateResult->resultCode,
				'errorCode' => $stateResult->errorCode,
				'errorCodeDescription' => $stateResult->errorCodeDescription );
		}
		$return = array( 'status' => true );
        $mids = array();
        foreach($stateResult->payoutState as $midKey => $state)
        {
            $mids[$midKey] = $state->mid;
        }
        if($mid && in_array($mid,$mids))
        {
            $fmid = array();
			foreach ( $stateResult->payoutState[$midKey] as $key => $entry )
			{
				if ( $key == 'mid' or $key == 'currency' )
				{
					$fmid[$key] = $entry;
				}
				else
				{
					$fmid[$key] = ( is_float( $entry ) ) ? number_format( $entry, 2,'.','' ) : $entry;
				}
			}
            return array_merge($return,$fmid);
        }
		foreach ( $stateResult->payoutState as $mid )
		{
			$fmid = array();
			foreach ( $mid as $key => $entry )
			{
				if ( $key == 'mid' or $key == 'currency' )
				{
					$fmid[$key] = $entry;
				}
				else
				{
					$fmid[$key] = ( is_float( $entry ) ) ? number_format( $entry, 2,'.','' ) : $entry;
				}
			}
			$return[] = $fmid;
		}
		return $return;
	}
	private function connect()
	{
		$this->soapInit();
		try
		{
			return new SoapClient( ( $this->productive ) ? $this->url['productive'] : $this->url['test'], $this->wsdl );
		}
		catch ( exception $e )
		{
			echo 'Exception: ', $e->getMessage(), "\n";
		}
	}
	private function generatePtid()
	{
		return 'test_' . date( 'Ymd-His' ) . '_' . rand( 1, 9999 );
	}
	private function soapInit()
	{
		if ( !class_exists( "SOAPClient" ) )
		{
			die( 'ERROR: SOAPClient isn´t enabled' );
		}
		$this->wsdl = array(
			'encoding' => 'utf-8',
			'connection_timeout' => 15,
			'features' => SOAP_SINGLE_ELEMENT_ARRAYS,
			'compression' => SOAP_COMPRESSION_GZIP );
		if ( !$this->productive )
		{
			ini_set( "soap.wsdl_cache_enabled", '0' );
			ini_set( "soap.wsdl_cache_ttl", '0' );
			$wsdl = array(
				'user_agent' => 'paysafecard payout class(testmode) v1.0',
				'trace' => true,
				'exceptions' => true,
				'cache_wsdl' => WSDL_CACHE_NONE );
		}
		else
		{
			$wsdl = array(
				'user_agent' => 'paysafecard payout class v1.0',
				'trace' => false,
				'exceptions' => false );
		}
		$this->wsdl = array_merge( $this->wsdl, $wsdl );
	}
	public function getDetails( $dump = false )
	{
		$debug = array(
			'transaction' => $this->transaction,
			'payoutResult' => $this->payoutResult,
			'soap' => $this->wsdl );
		if ( $dump )
		{
			echo '<pre>';
			var_dump( $debug );
			echo '</pre>';
		}
		return $debug;
	}
	private function setError( $errorCode )
	{
		switch ( $errorCode )
		{
			case '3167_20':
			case '3167_40':
            case '3167':
			case '3170_20':
			case '3170_40':
            case '3170':
			case '3194_20':
			case '3194_40':
            case '3194':
            case '3197':
            case '3198':
				$this->error['msg'] = 'Unfortunately there has been a problem executing your request. Please check your e-mail inbox for further information from paysafecard on how to solve the issue and try again.';
				break;
			case '3162':
				$this->error['msg'] = 'Unfortunately, no my paysafecard account exists under the e-mail address you have entered. Please check the address for a typing error. If you do not have a my paysafecard account, you can <a href="https://mypins.paysafecard.com/customer-registration/StartRegistration.xhtml" id="paysafecard_link">register for one online now for free</a>.';
				break;
			case '3195_20':
			case '3195_40':
            case '3195':
				$this->error['msg'] = 'The personal details associated with your my paysafecard account do not match the details of this account. Please check the first names, surnames and dates of birth entered in both accounts and request the payout again.';
				break;
			default:
				$this->error['msg'] = 'Sorry, the payout was not completed successfully! There was no money transfered. Please contact our customer support: support@company.com';
		}
		$this->error['code'] = $errorCode;
		return $this->error['msg'];
	}
}
